@@ -320,13 +320,19 @@ class ReceiveScreen(CScreen):
             status = self.app.wallet.get_request_status(self.screen.address)
             self.screen.status = _('Payment received') if status == PR_PAID else ''
 
+    def clear(self):
+        self.screen.address = ''
+        self.screen.amount = ''
+        self.screen.message = ''
+
     def get_new_address(self):
+        if not self.app.wallet:
+            return False
         addr = self.app.wallet.get_unused_address(None)
         if addr is None:
             return False
+        self.clear()
         self.screen.address = addr
-        self.screen.amount = ''
-        self.screen.message = ''
         return True
 
     def on_address(self, addr):
@@ -379,7 +385,7 @@ class ReceiveScreen(CScreen):
     def save_request(self):
         addr = str(self.screen.address)
         amount = str(self.screen.amount)
-        message = str(self.screen.message)
+        message = unicode(self.screen.message)
         amount = self.app.get_amount(amount) if amount else 0
         req = self.app.wallet.make_payment_request(addr, amount, message, None)
         self.app.wallet.add_payment_request(req, self.app.electrum_config)
@@ -472,12 +478,10 @@ class RequestsScreen(CScreen):
     kvname = 'requests'
 
     def update(self):
-
         self.menu_actions = [('Show', self.do_show), ('Details', self.do_view), ('Delete', self.do_delete)]
-
         requests_list = self.screen.ids.requests_container
         requests_list.clear_widgets()
-        _list = self.app.wallet.get_sorted_requests(self.app.electrum_config)
+        _list = self.app.wallet.get_sorted_requests(self.app.electrum_config) if self.app.wallet else []
         for req in _list:
             address = req['address']
             timestamp = req.get('time', 0)
